<link rel="import" href="../polymer/polymer.html">

<script>
/**
 * @polymerBehavior PalmettoBehaviors.PalmettoSharedBehavior
 **/
window.PalmettoBehaviors = window.PalmettoBehaviors || {};
PalmettoBehaviors.PalmettoSharedBehavior = PalmettoBehaviors.PalmettoSharedBehavior || {
  properties: {
    _selectedIncidentsUpdatedTriggered: {
      type: Boolean,
      value : true,
    },
    _palmettoTokenUpdatedTriggered: {
      type: Boolean,
      value : true,
    },
    _palmettoUserUpdatedTriggered: {
      type: Boolean,
      value : true,
    },
    configuration: {
        type: Object,
        notify: true,
        value: window.PALMETTO_CONFIGURATION
    },
    palmettoUserAccount: {
      type: Object,
      notify: true,
      value: function(){
        if(!window.palmettoUserAccount) {
          window.palmettoUserAccount = {};
        }
        return window.palmettoUserAccount;
      }
    },
    palmettoToken: {
      type: Object,
      notify: true,
      value: function() {
        if(!window.palmettoToken) {
          window.palmettoToken = {};
        }
        return window.palmettoToken;
      }
    },
    isHosted: {
        type: Boolean,
        value: false
    },
    isPalmettoApp : {
        type: Boolean,
        notify: true,
        value: true
    },
    appPermissions: {
      type: Object,
      notify: true,
      value: function() {
        if(!window.appPermissions) {
          window.appPermissions = {};
        }
        if(this.elementType && this.appPermissions.length > 0)
        {
          this.appPermissions.forEach(function(element){
              if(element.pvServiceName == this.elementType) {
                this.push("serviceActions",element);
                this.serviceAction = element.pvActionName;
                this.serviceID = element.pvServiceID
              }
            }.bind(this));
        }
        return window.appPermissions;
      }
    },
    serviceAction: {
      type: String,
      notify: true
    },
    serviceActions: {
        type: Array,
        value: []
    },
    serviceID: {
      type: String,
      notify: true,
      notify:true
    },
    selectedIncidents: {
      type: Array,
      notify: true,
      value: function() {
        if(!window.selectedIncidents) {
          window.selectedIncidents = [];
        }
        return window.selectedIncidents;
      }
    },
    // This is a hack and should be investigated further.
    shouldDispatchIncidentEvent: {
        type: Boolean,
        value: false,
    }
  },
  observers: [
    'tokenChanged(palmettoToken)',
    'selectedIncidentsChanged(selectedIncidents.splices)',
    'userAccountChanged(palmettoUserAccount)',
    'appPermissionsChanged(appPermissions)',
    'elementTypeChanged(elementType)',
  ],

	/**
	 * Iterates over the groups and returns the correct group with the groupID
	 * @param {Array} groups a list of the groups that the user has
	 * @param {Number} groupID the Id of the group we're looking for
	 */
	getGroup: function(groups, groupID) {
		for(var i=0; i < groups.length; i++) {
			if(groups[i].pvGroupID == groupID) {
				return groups[i];
			}
		}
	},

	/**
	 * This function occurs when the user logins and they have an activeGroup saved
	 * to the settings. This function then sets the relevent global variables and distributes the changes to the app
	 * @param {Event} e The custom event
	 */
	setActiveGroupFromSettings: function (e) {
		var group = this.getGroup(this.palmettoUserAccount.account2groups, e.detail.group);
		this.tempGroupName = group.pvGroupName;
		this.tempGroupID = e.detail.group;

		window.activeGroup = this.tempGroupID;
		window.activeGroupName = this.tempGroupName;

		this.importHref(this.resolveUrl("../palmetto-elements-settings/palmetto-sidebar-notification-driver.html"), function () {
		});
		this.set("currentGroupName", this.tempGroupName);
		this.fire("palmetto-group-changed", window.activeGroup);
		// this.set("selectedIncidents", []);
		// window.selectedIncidents =  [];
		// //Don't fire off the event to get the apps if we're parsing from the settings (login already handles that)
		//
		// // this.$.accountSettings.saveSetting("selectedIncidents", window.selectedIncidents);
		// window.dispatchEvent(new CustomEvent('selected-incidents-updated', { detail: undefined }));
		//

		if (document.querySelector("#group_" + this.tempGroupName))
			document.querySelector("#group_" + this.tempGroupName).style.backgroundColor = "rgba(76, 123, 252, 0.51)";
  },

  userAccountChanged: function(account) {
    if(window.palmettoUserAccount != account){
      window.palmettoUserAccount = account;
      window.dispatchEvent(new Event('palmetto-useraccount-updated'));
    }
  },
  //This code is duplicated due to the complexities of switching between
  //normally hosted apps and standalone apps
  appPermissionsChanged: function(permissions) {
    if(window.appPermissions != permissions){
      window.appPermissions = permissions;
      window.dispatchEvent(new Event('palmetto-apppermissions-updated'), { detail: permissions });

      if(this.elementType && this.appPermissions.length > 0)
      {
        this.appPermissions.forEach(function(element){
            if(element.pvServiceName == this.elementType) {
              this.push("serviceActions",element);
              this.serviceAction = element.pvActionName;
              this.serviceID = element.pvServiceID
            }
          }.bind(this));
      }
    }
  },
  elementTypeChanged: function(elementType) {
      if(this.elementType && this.appPermissions.length > 0)
      {
        this.appPermissions.forEach(function(element){
            if(element.pvServiceName == this.elementType) {
              this.push("serviceActions",element);
              this.serviceAction = element.pvActionName;
              this.serviceID = element.pvServiceID
            }
          }.bind(this));
      }
  },
  tokenChanged: function(palmettoToken) {
    // if(this._palmettoTokenUpdatedTriggered) {
      // this.set('palmettoTokenUpdatedTriggered', false);
      // return;
    // }
    // When the token is changed, we need to update the window then notify all other shared
    // behaviors.
    if(window.palmettoToken != palmettoToken){
      window.palmettoToken = palmettoToken;
      window.dispatchEvent(new Event('palmetto-token-updated'));
      window.dispatchEvent(new CustomEvent('palmetto-token-updated', { detail: palmettoToken }));
    }
  },
  _selectedIncidentsChange: function(){},
  selectedIncidentsChanged: function(selectedIncidents) {
    if(this._selectedIncidentsUpdatedTriggered) {
      this.set('_selectedIncidentsUpdatedTriggered', false);
      return;
    }
    // When the selected incidents are changed, we need to update the window then notify
    // all other shared behaviors.

    this.selectedIncidents.forEach(function(incident, index) {
        if(incident.pvParentIncidentID) {
            this.selectedIncidents.splice(index, 1)
        }
    }.bind(this))

    if(window.selectedIncidents != this.selectedIncidents){
      window.selectedIncidents = this.selectedIncidents;
    }
    // This is a hack. This should be changed.
    if(this.shouldDispatchIncidentEvent) {
      window.dispatchEvent(new CustomEvent('selected-incidents-updated', { detail: selectedIncidents }));
      this._selectedIncidentsChange(this.selectedIncidents);
      this.set('shouldDispatchIncidentEvent', false);
    }
  },
  incidentUpdate: function(incidents){
    this.fire('incidentUpdated', incidents);
  },

  ready: function() {
    window.addEventListener('selected-incidents-updated', function(event) {

      this.debounce('wait-for-multiple-incidents-changes', function() {
        this.set('_selectedIncidentsUpdatedTriggered', true);
  
          // just to make sure merged incidents dont show up in the list
        window.selectedIncidents.forEach(function(incident, index) {
            if(incident.pvParentIncidentID) {
                window.selectedIncidents.splice(index, 1)
            }
        }.bind(this))
  
  
        this.set('selectedIncidents', JSON.parse(JSON.stringify(window.selectedIncidents)));
  
        if(this.incidentsUpdatedFunc) {
          this.incidentsUpdatedFunc();
        }
      }.bind(this), 2000)

    }.bind(this));
    window.addEventListener('palmetto-useraccount-updated',function() {
      this.set("palmettoUserAccount",window.palmettoUserAccount);
    }.bind(this));
    window.addEventListener('palmetto-token-updated', function() {
      this.set('_palmettoTokenUpdatedTriggered', true);
      this.set('palmettoToken', window.palmettoToken);
    }.bind(this));
    window.addEventListener('palmetto-user-updated', function() {
      this.set('_palmettoUserUpdatedTriggered', true);
      this.set('palmettoUser', window.palmettoUser);
    }.bind(this));
    window.addEventListener('palmetto-apppermissions-updated', function() {
      this.set('_palmettoPermissionsUpdateTriggered', true);
      this.set('appPermissions', window.appPermissions);
    }.bind(this));
    window.addEventListener("appPermissions",function(e){
      if(e.detail)
        this.appPermissions = e.detail;
      //parse the permissions based on the current application
      if(this.elementType)
      {
        this.appPermissions.forEach(function(element){
            if(element.pvServiceName == this.elementType) {
              this.push("serviceActions",element);
              this.serviceAction = element.pvActionName;
              this.serviceID = element.pvServiceID
            }
          }.bind(this));
      }
    }.bind(this));
  },
};
</script>
